from typing import Sequence, Optional

from puppeter.domain.model import Installer
from puppeter.domain.model.configurer import Configurer


class InstallerGateway:
    def configurers(self, installer: Optional[Installer]) -> Sequence[Configurer]: ...
    def _provide_install_configurers(self, installer: Installer) -> Sequence[Configurer]: ...
    def _puppet_cert_issue(self, installer: Installer) -> Sequence[Configurer]: ...
    def _puppet_services(self, installer: Installer) -> Sequence[Configurer]: ...
    def _puppet_conf(self, installer: Installer) -> Sequence[Configurer]: ...

